/* Copyright (c) 1998, MpegTV LLC */
#ifndef _sih_api_h
#define _sih_api_h
#ifndef lint
static const char rcsid_sih_api_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/sih_api.h,v 1.28 1999/04/11 03:54:09 tristan Exp tristan $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "misc.h"
#include "msg.h"
#include "xfile.h"

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
/* current version */
#define SIH_VERSION	0

/*
 * SIH_read may return -1 with errno set to SIH_READ_ERROR_TOO_SMALL to
 * indicate that the count passed is too small. This feature is very useful
 * for example when reading UDP sockets.  When reading a UDP socket, the
 * read count passed to the 'read' routine should always be at least
 * the maximum UDP packet size expected, otherwise some data can be lost.
 * SIH_READ_ERROR_TOO_SMALL can therefore be returned if SIH_read is
 * called with a 'count' value smaller than the maximum UDP packet size expected.
 * Note that the packet payload size should be larger than this maximum
 * UDP packet size expected, otherwise SIH_read would always be called with
 * a count too small!
 */
#define SIH_READ_ERROR_TOO_SMALL	(-1)

#if 0 /* Tristan Savatier - Sun Aug 23 23:48:12 PDT 1998 */
///* sih_callback() is now a macro */
//#define sih_feedback(data, size) (*SIH_callbacks.sih_feedback_ptr)(data, size)
#endif /* 0 - Tristan Savatier - Sun Aug 23 23:48:13 PDT 1998 */

/*
 * Handler id of built-in handlers.
 */
#define SIH_HANDLER_ID_FILE			255
#define SIH_HANDLER_ID_FD			254
#define SIH_HANDLER_ID_VCD			253
#define SIH_HANDLER_ID_PORT			252
#define SIH_HANDLER_ID_HTTP_FTP		251

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

/* to specify configuration of input buffer pool */
typedef struct {
	u_int	packet_count;		/* packet count */
	u_int	packet_gross_size;	/* packet total size */
	u_int	packet_payload_size;	/* packet payload size */
	u_int	packet_offset;		/* packet header size (i.e. offset to payload) */
	
	/* following are experimental */
	
#if 0 /* Tristan Savatier - Mon Mar 16 18:29:23 PST 1998 */
//	u_int	free_count_trigger;	/* dont start reading unless # free packets > this (default 0) */
#endif /* 0 - Tristan Savatier - Mon Mar 16 18:29:23 PST 1998 */
#if 0 /* Tristan Savatier - Tue Feb 24 15:43:54 PST 1998 */
//	u_int	list_count_trigger;	/* experimental (default 0) */
#endif /* 0 - Tristan Savatier - Tue Feb 24 15:43:54 PST 1998 */
} SIB_IPL_SIZES;

/* callback routines etc */
typedef struct sih_resources SIH_RESOURCES;

struct sih_resources {
	MSG_STRUCT *msg;
	const void *z;
	int handler_id;
	int (*feedback)(const SIH_RESOURCES *sih_resources, const void *data, size_t size);
};

/* Stream Input Interface (SIH), used by our SIB_IPL layer */
typedef struct {
	int	SIH_version;	/* handler must set this to SIH_VERSION */
	void *(*SIH_init)(const SIH_RESOURCES *resources, const char *p);
	int (*SIH_free)(void *handler_data);
	int	(*SIH_probe_url)(const SIH_RESOURCES *resources, const char *url);
	void *(*SIH_open_url)(void *handler_data, const char *url);
	int (*SIH_close)(void *stream_data);
	int (*SIH_read)(void *stream_data, char *buf, size_t count);
	off_t (*SIH_lseek)(void *stream_data, off_t offset, int whence);
	int (*SIH_cntl)(void *handler_data, const void *data, u_int size);
	XFILE_HANDLE (*SIH_fd)(void *stream_data);
	off_t (*SIH_size)(void *stream_data);
	void (*SIH_buffer)(void *stream_data, SIB_IPL_SIZES *sizes);
} SIH_EXPOSED;

#if 0 /* Tristan Savatier - Sun Aug 23 01:24:27 PDT 1998 */
//typedef struct {
//	int handler_id;
//} SIH_HANDLER_ID;
#endif /* 0 - Tristan Savatier - Sun Aug 23 01:24:27 PDT 1998 */

typedef int SIH_HANDLER_ID;

/* block of data send received by application with A2G_COM_SIH_FEEDBACK */
typedef struct {
	int handler_id;
	char data[1];	/* this is a dummy declaration. in fact data may be any size */
} SIH_FEEDBACK;


/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

#if 0 /* Tristan Savatier - Sun Aug 16 20:40:03 PDT 1998 */
//extern SIH_CALLBACKS SIH_callbacks;
#endif /* 0 - Tristan Savatier - Sun Aug 16 20:40:03 PDT 1998 */

extern int _init(void);
extern int _fini(void);
extern int	SIH_register(SIH_EXPOSED *exposed, int version);

#ifdef __cplusplus
}
#endif

#endif /* _sih_api_h */
